import { demoTrace, demoConfig } from "./demo-config";
import "@vanillawc/wc-codemirror";

import { default as jsyaml } from "js-yaml";

import { automationGraph } from "./hat-graph/make-graph";
import { html } from "lit-element";
import { render } from "lit-html";
import { mdiBowTie } from "@mdi/js";
let selectedNode;

function nodeSelected(ev) {
  if (selectedNode) {
    selectedNode.removeAttribute("selected");
  }
  ev.detail.node.setAttribute("selected", "");
  selectedNode = ev.detail.node;

  const snippet = document.querySelector("#snippet");
  snippet.querySelector("span").innerHTML = ev.detail.path;
  const code = snippet.querySelector("wc-codemirror");
  code.value = jsyaml.dump(ev.detail.config);
  code.currentNode = ev.target;
}

function rebuildGraph(config = undefined) {
  const graphCard = document.querySelector("#graph");

  if (!config) {
    const codeEl = document.querySelector("#fullcode wc-codemirror");
    config = jsyaml.load(codeEl.value);
  }

  while (graphCard.firstChild) graphCard.removeChild(graphCard.firstChild);

  const traceEl = document.querySelector("#trace wc-codemirror");
  const trace = jsyaml.load(traceEl.value);

  const graph = automationGraph(config, trace);

  graph.addEventListener("node-selected", nodeSelected);

  graphCard.appendChild(graph);
}

function setup_snippet_editor() {
  document.querySelector("#saveSnippet").onclick = () => {
    const code = document.querySelector("#snippet");
    if (code.currentNode) {
      code.currentNode.updateNode(jsyaml.load(code.value));
    }
  };
  document.querySelector("#deleteSnippet").onclick = () => {
    const code = document.querySelector("#snippet");
    if (code.currentNode) {
      code.currentNode.deleteNode();
    }
  };
}

function setup_trace_editor() {
  const src = demoTrace;
  const trace = document.querySelector("#trace");
  window.setTimeout(
    () => (trace.querySelector("wc-codemirror").value = jsyaml.dump(src)),
    100
  );
  trace.querySelector("button").addEventListener("click", () => rebuildGraph());
}

function setup_fullcode_editor() {
  const src = demoConfig;
  const code = document.querySelector("#fullcode");

  window.setTimeout(
    () => (code.querySelector("wc-codemirror").value = jsyaml.dump(src)),
    100
  );

  code.querySelector("button").addEventListener("click", () => rebuildGraph());
}

function setup_trigger_buttons() {
  const btn = document.querySelector("#triggers");
  btn.querySelectorAll("button").forEach((b, i) => {
    b.addEventListener("click", () => {
      console.log(i, "clicked");
      btn.dispatchEvent(
        new CustomEvent("event-triggered", {
          detail: { trigger: `trigger${i + 1}` },
          bubbles: true,
        })
      );
    });
  });

  btn.querySelectorAll("input").forEach((c, i) => {
    c.addEventListener("change", () => {
      const condition = ["a", "b", "c"][i];
      console.log(condition, "changed");
      btn.dispatchEvent(
        new CustomEvent("states-changed", {
          detail: {
            state: {
              a: btn.querySelector("#a").checked,
              b: btn.querySelector("#b").checked,
              c: btn.querySelector("#c").checked,
            },
          },
          bubbles: true,
        })
      );
    });
  });
}

function setup() {
  setup_snippet_editor();

  setup_fullcode_editor();
  setup_trace_editor();
  setup_trigger_buttons();

  window.setTimeout(() => rebuildGraph(), 300);
}

setup();
